import os
import sys

# Setting the seeds to get reproducible results
os.environ['PYTHONHASHSEED'] = '0'

# we need to import python modules from the $SUMO_HOME/tools directory
try:
    sys.path.append(os.path.join(os.path.dirname(
        __file__), '..', '..', '..', '..', "tools"))  # tutorial in tests
    sys.path.append(os.path.join(os.environ.get("SUMO_HOME", os.path.join(
        os.path.dirname(__file__), "..", "..", "..")), "tools"))  # tutorial in docs
    from sumolib import checkBinary
except ImportError:
    sys.exit(
        "please declare environment variable 'SUMO_HOME' as the root directory of your sumo installation (it should contain folders 'bin', 'tools' and 'docs')")

import traci
import traci.constants as tc


gui = True
if gui:
    sumoBinary = checkBinary('sumo-gui')
else:
    sumoBinary = checkBinary('sumo')
config_path = os.path.dirname(__file__)+"/../../../Data/StraightRoad.sumocfg"


def close(self):
    traci.close()


def start(gui=False):
    sumoBinary = checkBinary('sumo-gui') if gui else checkBinary('sumo')
    traci.start([sumoBinary, "-c", config_path])

if __name__ == "__main__":
    AutoCarID = 'Auto'
    laneChangeParameter = 0.8
    start(True)
    AutoCarAvailable = False
    while AutoCarAvailable == False:
        traci.simulationStep()
        VehicleIds = traci.vehicle.getIDList()
        if AutoCarID in traci.vehicle.getIDList():
            AutoCarAvailable = True
            for VehId in VehicleIds:
                traci.vehicle.setLaneChangeMode(VehId, 0b010001010101)
                if VehId == AutoCarID:
                    traci.vehicle.setLaneChangeMode(VehId, 0)  # Disable automatic lane changing
                    traci.vehicle.setSpeedMode(VehId, 0)
    for timesteps in range(10000):
        VehicleIds = traci.vehicle.getIDList()
        if AutoCarID in traci.vehicle.getIDList():
            for VehId in VehicleIds:
                if VehId != AutoCarID:
                    traci.vehicle.setLaneChangeMode(VehId, 0b010001010101) # For future vehicles that are created
            laneindex = traci.vehicle.getLaneIndex(AutoCarID)
            positionInLane = traci.vehicle.getLateralLanePosition(AutoCarID)

            if laneindex == 0 and positionInLane < -0.6:
                laneChangeParameter = 0.8
            if laneindex == 1 and positionInLane > 0.6:
                laneChangeParameter = -0.8
            traci.vehicle.changeSublane(AutoCarID, laneChangeParameter)
            traci.simulationStep()
        else:
            print("Collision occured")
            break

